@@ -9,6 +9,7 @@ from beem.account import Account
 from beem.nodelist import NodeList
 from beem.utils import formatTimeString, addTzInfo
 from beem.blockchain import Blockchain
+from beem.exceptions import WrongMasterPasswordException
 from steemengine.api import Api
 from steemengine.tokens import Tokens
 from steemengine.tokenobject import Token
@@ -39,9 +40,11 @@ except ImportError:
     KEYRING_AVAILABLE = False
 
 
-def unlock_wallet(stm, password=None):
+def unlock_wallet(stm, password=None, allow_wif=True):
     if stm.unsigned and stm.nobroadcast:
         return True
+    if not stm.wallet.locked():
+        return True
     password_storage = stm.config["password_storage"]
     if not password and KEYRING_AVAILABLE and password_storage == "keyring":
         password = keyring.get_password("beem", "wallet")
@@ -50,8 +53,22 @@ def unlock_wallet(stm, password=None):
     if bool(password):
         stm.wallet.unlock(password)
     else:
-        password = click.prompt("Password to unlock wallet", confirmation_prompt=False, hide_input=True)
-        stm.wallet.unlock(password)
+        if allow_wif:
+            password = click.prompt("Password to unlock wallet or posting/active wif", confirmation_prompt=False, hide_input=True)
+        else:
+            password = click.prompt("Password to unlock wallet", confirmation_prompt=False, hide_input=True)
+        try:
+            stm.wallet.unlock(password)
+        except:
+            try:
+                stm.wallet.setKeys([password])
+                print("Wif accepted!")
+                return True                
+            except:
+                if allow_wif:
+                    raise WrongMasterPasswordException("entered password is not a valid password/wif")
+                else:
+                    raise WrongMasterPasswordException("entered password is not a valid password")
 
     if stm.wallet.locked():
         if password_storage == "keyring" or password_storage == "environment":
@@ -70,12 +87,14 @@ def unlock_wallet(stm, password=None):
 
 
 @click.group(chain=True)
+@click.option(
+    '--node', '-n', default="https://api.steemit.com", help="URL for public Steem API (e.g. https://api.steemit.com)")
 @click.option(
     '--no-broadcast', '-d', is_flag=True, default=False, help="Do not broadcast")
 @click.option(
     '--verbose', '-v', default=3, help='Verbosity')
 @click.version_option(version=__version__)
-def cli(no_broadcast, verbose):
+def cli(node, no_broadcast, verbose):
     # Logging
     log = logging.getLogger(__name__)
     verbosity = ["critical", "error", "warn", "info", "debug"][int(
@@ -89,6 +108,7 @@ def cli(no_broadcast, verbose):
     log.addHandler(ch)
     debug = verbose > 0
     stm = Steem(
+        node=node,
         nobroadcast=no_broadcast,
         )
     set_shared_blockchain_instance(stm)    
@@ -660,9 +680,10 @@ def sell(amount, token, price, account):
 
 @cli.command()
 @click.argument('order_type', nargs=1)
-@click.argument('order_id', nargs=1)
+@click.argument('order_id', nargs=1, required=False)
 @click.option('--account', '-a', help='Buy with this account (defaults to "default_account")')
-def cancel(order_type, order_id, account):
+@click.option('--yes', '-y', help='Answer yes to all questions', is_flag=True, default=False)
+def cancel(order_type, order_id, account, yes):
     """Cancel a buy/sell order
     
         order_type is either sell or buy
@@ -678,13 +699,49 @@ def cancel(order_type, order_id, account):
     market = Market(steem_instance=stm)
     if not unlock_wallet(stm):
         return
+    if order_id is None and order_type == "buy":
+        wallet = Wallet(account, steem_instance=stm)
+        for t in wallet:
+            token = t["symbol"]
+            buy_book = market.get_buy_book(token, account)
+            sorted_buy_book = sorted(buy_book, key=lambda account: float(account["price"]), reverse=True)    
+            for order in sorted_buy_book:
+                print("Cancel sell order with id: %s for %.8f %s at %s" % (order["_id"], float(order["quantity"]), token, float(order["price"])))
+                if not yes:
+                    ret = input("continue [y/n]?")
+                    if ret not in ["y", "yes"]:
+                        continue            
+                tx = market.cancel(account, order_type, int(order_id))
+                tx = json.dumps(tx, indent=4)
+                print(tx)
+                if yes:
+                    time.sleep(4)
+        return
+    elif order_id is None and order_type == "sell":
+        wallet = Wallet(account, steem_instance=stm)
+        for t in wallet:
+            token = t["symbol"]
+            sell_book = market.get_sell_book(token, account)
+            sorted_sell_book = sorted(sell_book, key=lambda account: float(account["price"]), reverse=True)    
+            for order in sorted_sell_book:
+                print("Cancel sell order with id: %s for %.8f %s at %s" % (order["_id"], float(order["quantity"]), token, float(order["price"])))
+                if not yes:
+                    ret = input("continue [y/n]?")
+                    if ret not in ["y", "yes"]:
+                        continue            
+                tx = market.cancel(account, order_type, int(order_id))
+                tx = json.dumps(tx, indent=4)
+                print(tx)
+                if yes:
+                    time.sleep(4)                
+        return        
     tx = market.cancel(account, order_type, int(order_id))
     tx = json.dumps(tx, indent=4)
     print(tx)
 
 
 @cli.command()
-@click.argument('token', nargs=1)
+@click.argument('token', nargs=1, required=False)
 @click.option('--account', '-a', help='Buy with this account (defaults to "default_account")')
 def buybook(token, account):
     """Returns the buy book for the given token
@@ -697,17 +754,31 @@ def buybook(token, account):
         print("Please set a Steem node")
         return
     market = Market(steem_instance=stm)
-    buy_book = market.get_buy_book(token, account)
-    sorted_buy_book = sorted(buy_book, key=lambda account: float(account["price"]), reverse=True)
-    t = PrettyTable(["order_id", "account", "quantity", "price"])
-    t.align = "l"
-    for order in sorted_buy_book:
-        t.add_row([order["_id"], order["account"], order["quantity"], order["price"]])
-    print(t.get_string())    
+    if token is None:
+        if account is None:
+            account = stm.config["default_account"]        
+        wallet = Wallet(account, steem_instance=stm)
+        table = PrettyTable(["token", "order_id", "account", "quantity", "price"])
+        table.align = "l"        
+        for t in wallet:
+            token = t["symbol"]
+            buy_book = market.get_buy_book(token, account)
+            sorted_buy_book = sorted(buy_book, key=lambda account: float(account["price"]), reverse=True)    
+            for order in sorted_buy_book:
+                table.add_row([token, order["_id"], order["account"], order["quantity"], order["price"]])   
+        print(table.get_string())   
+    else:    
+        buy_book = market.get_buy_book(token, account)
+        sorted_buy_book = sorted(buy_book, key=lambda account: float(account["price"]), reverse=True)
+        t = PrettyTable(["order_id", "account", "quantity", "price"])
+        t.align = "l"
+        for order in sorted_buy_book:
+            t.add_row([order["_id"], order["account"], order["quantity"], order["price"]])
+        print(t.get_string())    
 
 
 @cli.command()
-@click.argument('token', nargs=1)
+@click.argument('token', nargs=1, required=False)
 @click.option('--account', '-a', help='Buy with this account (defaults to "default_account")')
 def sellbook(token, account):
     """Returns the sell book for the given token
@@ -719,13 +790,27 @@ def sellbook(token, account):
         print("Please set a Steem node")
         return
     market = Market(steem_instance=stm)
-    sell_book = market.get_sell_book(token, account)
-    sorted_sell_book = sorted(sell_book, key=lambda account: float(account["price"]), reverse=False)
-    t = PrettyTable(["order_id", "account", "quantity", "price"])
-    t.align = "l"
-    for order in sorted_sell_book:
-        t.add_row([order["_id"], order["account"], order["quantity"], order["price"]])
-    print(t.get_string())  
+    if token is None:
+        if account is None:
+            account = stm.config["default_account"]        
+        wallet = Wallet(account, steem_instance=stm)
+        table = PrettyTable(["token", "order_id", "account", "quantity", "price"])
+        table.align = "l"        
+        for t in wallet:
+            token = t["symbol"]
+            sell_book = market.get_sell_book(token, account)
+            sorted_sell_book = sorted(sell_book, key=lambda account: float(account["price"]), reverse=False)  
+            for order in sorted_sell_book:
+                table.add_row([token, order["_id"], order["account"], order["quantity"], order["price"]])   
+        print(table.get_string())   
+    else:    
+        sell_book = market.get_sell_book(token, account)
+        sorted_sell_book = sorted(sell_book, key=lambda account: float(account["price"]), reverse=False)
+        t = PrettyTable(["order_id", "account", "quantity", "price"])
+        t.align = "l"
+        for order in sorted_sell_book:
+            t.add_row([order["_id"], order["account"], order["quantity"], order["price"]])
+        print(t.get_string())  
 
 
 if __name__ == "__main__":
